/*
 * Copyright (C) 2016 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dico;

//Import packages
import dico.Build;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.Deflater;
import java.util.zip.DeflaterOutputStream;

public class Exec extends JFrame {

	static JFrame frmMain;
	static Container pane;
	static JProgressBar barDo;
	static String language;
	static String inFile;
	static Build parent;
	private static JButton btnCancel;

	public static void exec() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException |
			InstantiationException |
			IllegalAccessException |
			UnsupportedLookAndFeelException e) {
		}

		//Create all components
		frmMain.setUndecorated(true);
		frmMain.getRootPane().setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLoweredBevelBorder(),
			BorderFactory.createRaisedBevelBorder()));
		frmMain.setTitle(I18N.getMsg("dlg.build.build.progress"));
		frmMain.setLocationRelativeTo(null);
		frmMain.setSize(310, 100); //Window size 300x100 pixels
		pane = frmMain.getContentPane();
		pane.setLayout(null); //Use the null layout
		frmMain.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); //Exit when X is clicked
		btnCancel = new JButton("Cancel");
		barDo = new JProgressBar(0, 100); //Min value: 0 Max value: 100	

		//Add components to pane
		pane.add(btnCancel);
		pane.add(barDo);

		//Position controls (X, Y, width, height)
		barDo.setBounds(10, 10, 280, 20);
		btnCancel.setBounds(100, 35, 100, 25);

		//Make frame visible
		frmMain.setResizable(false); //No resize
		frmMain.setVisible(true);

		//Add action listeners
		btnCancel.addActionListener(new btnCancelAction()); //Add the button's action
		new Thread(new TheTread()).start();
	}

	Exec(Build p,String code, String inFile) {
		Exec.parent=p;
		Exec.language = code;
		Exec.inFile = inFile;
		frmMain=this;
	}

	//The action
	public static class btnCancelAction implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			System.exit(0);
		}

	}

	//The thread
	public static class TheTread implements Runnable {

		@Override
		@SuppressWarnings("SleepWhileInLoop")
		public void run() {
			frmMain.setTitle(I18N.getMsg("dlg.build.build.progress"));
			barDo.setMaximum(WIDTH);
			try {
				File txtFile = new File(inFile);
				barDo.setMaximum(100);
				//create the input stream
				FileInputStream fis;
				fis = new FileInputStream(txtFile);
				Reader reader;
				reader = new InputStreamReader(fis, "UTF8");
				BufferedReader txt = new BufferedReader(reader);
				//read all the words
				int curent = 0;
				ArrayList<String> wordList = new ArrayList<>();
				String word;
				long total = txtFile.length();
				word = txt.readLine();
				barDo.setStringPainted(true);
				barDo.setMaximum((int) total);
				while (word != null) {
					curent += (word.length()+1);
					barDo.setString(curent+"/"+total);
					barDo.setValue(curent);
					barDo.repaint();
					wordList.add(word);
					word = txt.readLine();
					Thread.sleep(1);
				}
				//Save dictionary
				//Create the output stream
				String outFile = inFile.substring(0, inFile.lastIndexOf("/")) + "/dictionary_" + language + ".ortho";
				File dictFile = new File(outFile);
				OutputStream dict;
				dict = new FileOutputStream(dictFile);
				dict = new BufferedOutputStream(dict);
				Deflater deflater = new Deflater();
				deflater.setLevel(Deflater.BEST_COMPRESSION);
				dict = new DeflaterOutputStream(dict, deflater);
				dict = new BufferedOutputStream(dict);
				//create the input stream
				PrintStream dictPs;
				dictPs = new PrintStream(dict, false, "UTF8");
				barDo.setMaximum(wordList.size());
				frmMain.setTitle(I18N.getMsg("dlg.build.build.save"));
				String[] words = wordList.toArray(new String[wordList.size()]);
				Arrays.sort(words);
				curent = 0;
				total=words.length;
				barDo.setMaximum((int) total);
				for (String w : words) {
					curent++;
					barDo.setString(curent+"/"+total);
					barDo.setValue(curent);
					barDo.repaint(); //Refresh graphics
					dictPs.print(w + '\n');
					Thread.sleep(1);
				}
				System.exit(0);
			} catch (FileNotFoundException |
				UnsupportedEncodingException |
				InterruptedException ex) {
				Logger.getLogger(Exec.class.getName()).log(Level.SEVERE, null, ex);
			} catch (IOException ex) {
				Logger.getLogger(Exec.class.getName()).log(Level.SEVERE, null, ex);
			}
		}
	}
}
